@@ -1,11 +1,11 @@
+//go:build !yq_nolua
+
 package yqlib
 
 import (
 	"fmt"
 	"io"
 	"strings"
-
-	yaml "gopkg.in/yaml.v3"
 )
 
 type luaEncoder struct {
@@ -80,10 +80,10 @@ func (le *luaEncoder) PrintLeadingContent(writer io.Writer, content string) erro
 	return nil
 }
 
-func (le *luaEncoder) encodeString(writer io.Writer, node *yaml.Node) error {
+func (le *luaEncoder) encodeString(writer io.Writer, node *CandidateNode) error {
 	quote := "\""
 	switch node.Style {
-	case yaml.LiteralStyle, yaml.FoldedStyle, yaml.FlowStyle:
+	case LiteralStyle, FoldedStyle, FlowStyle:
 		for i := 0; i < 10; i++ {
 			if !strings.Contains(node.Value, "]"+strings.Repeat("=", i)+"]") {
 				err := writeString(writer, "["+strings.Repeat("=", i)+"[\n")
@@ -97,7 +97,7 @@ func (le *luaEncoder) encodeString(writer io.Writer, node *yaml.Node) error {
 				return writeString(writer, "]"+strings.Repeat("=", i)+"]")
 			}
 		}
-	case yaml.SingleQuotedStyle:
+	case SingleQuotedStyle:
 		quote = "'"
 
 		// fallthrough to regular ol' string
@@ -116,7 +116,7 @@ func (le *luaEncoder) writeIndent(writer io.Writer) error {
 	return writeString(writer, strings.Repeat(le.indentStr, le.indent))
 }
 
-func (le *luaEncoder) encodeArray(writer io.Writer, node *yaml.Node) error {
+func (le *luaEncoder) encodeArray(writer io.Writer, node *CandidateNode) error {
 	err := writeString(writer, "{")
 	if err != nil {
 		return err
@@ -127,7 +127,7 @@ func (le *luaEncoder) encodeArray(writer io.Writer, node *yaml.Node) error {
 		if err != nil {
 			return err
 		}
-		err := le.Encode(writer, child)
+		err := le.encodeAny(writer, child)
 		if err != nil {
 			return err
 		}
@@ -179,7 +179,7 @@ func needsQuoting(s string) bool {
 	return false
 }
 
-func (le *luaEncoder) encodeMap(writer io.Writer, node *yaml.Node, global bool) error {
+func (le *luaEncoder) encodeMap(writer io.Writer, node *CandidateNode, global bool) error {
 	if !global {
 		err := writeString(writer, "{")
 		if err != nil {
@@ -190,7 +190,7 @@ func (le *luaEncoder) encodeMap(writer io.Writer, node *yaml.Node, global bool)
 	for i, child := range node.Content {
 		if (i % 2) == 1 {
 			// value
-			err := le.Encode(writer, child)
+			err := le.encodeAny(writer, child)
 			if err != nil {
 				return err
 			}
@@ -261,13 +261,13 @@ func (le *luaEncoder) encodeMap(writer io.Writer, node *yaml.Node, global bool)
 	return writeString(writer, "}")
 }
 
-func (le *luaEncoder) encodeAny(writer io.Writer, node *yaml.Node) error {
+func (le *luaEncoder) encodeAny(writer io.Writer, node *CandidateNode) error {
 	switch node.Kind {
-	case yaml.SequenceNode:
+	case SequenceNode:
 		return le.encodeArray(writer, node)
-	case yaml.MappingNode:
+	case MappingNode:
 		return le.encodeMap(writer, node, false)
-	case yaml.ScalarNode:
+	case ScalarNode:
 		switch node.Tag {
 		case "!!str":
 			return le.encodeString(writer, node)
@@ -280,8 +280,7 @@ func (le *luaEncoder) encodeAny(writer io.Writer, node *yaml.Node) error {
 			return writeString(writer, strings.ToLower(node.Value))
 		case "!!int":
 			if strings.HasPrefix(node.Value, "0o") {
-				var octalValue int
-				err := node.Decode(&octalValue)
+				_, octalValue, err := parseInt64(node.Value)
 				if err != nil {
 					return err
 				}
@@ -290,7 +289,7 @@ func (le *luaEncoder) encodeAny(writer io.Writer, node *yaml.Node) error {
 			return writeString(writer, strings.ToLower(node.Value))
 		case "!!float":
 			switch strings.ToLower(node.Value) {
-			case ".inf":
+			case ".inf", "+.inf":
 				return writeString(writer, "(1/0)")
 			case "-.inf":
 				return writeString(writer, "(-1/0)")
@@ -300,29 +299,32 @@ func (le *luaEncoder) encodeAny(writer io.Writer, node *yaml.Node) error {
 				return writeString(writer, node.Value)
 			}
 		default:
-			return fmt.Errorf("Lua encoder NYI -- %s", node.ShortTag())
+			return fmt.Errorf("Lua encoder NYI -- %s", node.Tag)
 		}
-	case yaml.DocumentNode:
-		if le.globals {
-			if node.Content[0].Kind != yaml.MappingNode {
-				return fmt.Errorf("--lua-global requires a top level MappingNode")
-			}
-			return le.encodeMap(writer, node.Content[0], true)
-		}
-		err := writeString(writer, le.docPrefix)
-		if err != nil {
-			return err
-		}
-		err = le.encodeAny(writer, node.Content[0])
-		if err != nil {
-			return err
-		}
-		return writeString(writer, le.docSuffix)
 	default:
-		return fmt.Errorf("Lua encoder NYI -- %s", node.ShortTag())
+		return fmt.Errorf("Lua encoder NYI -- %s", node.Tag)
 	}
 }
 
-func (le *luaEncoder) Encode(writer io.Writer, node *yaml.Node) error {
-	return le.encodeAny(writer, node)
+func (le *luaEncoder) encodeTopLevel(writer io.Writer, node *CandidateNode) error {
+	err := writeString(writer, le.docPrefix)
+	if err != nil {
+		return err
+	}
+	err = le.encodeAny(writer, node)
+	if err != nil {
+		return err
+	}
+	return writeString(writer, le.docSuffix)
+}
+
+func (le *luaEncoder) Encode(writer io.Writer, node *CandidateNode) error {
+
+	if le.globals {
+		if node.Kind != MappingNode {
+			return fmt.Errorf("--lua-global requires a top level MappingNode")
+		}
+		return le.encodeMap(writer, node, true)
+	}
+	return le.encodeTopLevel(writer, node)
 }
