@@ -10,7 +10,7 @@ var sortKeysOperatorScenarios = []expressionScenario{
 		document:    `{c: frog, a: blah, b: bing}`,
 		expression:  `sort_keys(.)`,
 		expected: []string{
-			"D0, P[], (doc)::{a: blah, b: bing, c: frog}\n",
+			"D0, P[], (!!map)::{a: blah, b: bing, c: frog}\n",
 		},
 	},
 	{
@@ -18,7 +18,7 @@ var sortKeysOperatorScenarios = []expressionScenario{
 		document:   `{c: frog}`,
 		expression: `sort_keys(.d)`,
 		expected: []string{
-			"D0, P[], (doc)::{c: frog}\n",
+			"D0, P[], (!!map)::{c: frog}\n",
 		},
 	},
 	{
