@@ -7,22 +7,16 @@ func referenceOperator(d *dataTreeNavigator, context Context, expressionNode *Ex
 }
 
 func valueOperator(d *dataTreeNavigator, context Context, expressionNode *ExpressionNode) (Context, error) {
-	log.Debug("value = %v", expressionNode.Operation.CandidateNode.Node.Value)
+	log.Debug("value = %v", expressionNode.Operation.CandidateNode.Value)
 	if context.MatchingNodes.Len() == 0 {
-		clone, err := expressionNode.Operation.CandidateNode.Copy()
-		if err != nil {
-			return Context{}, err
-		}
+		clone := expressionNode.Operation.CandidateNode.Copy()
 		return context.SingleChildContext(clone), nil
 	}
 
 	var results = list.New()
 
 	for el := context.MatchingNodes.Front(); el != nil; el = el.Next() {
-		clone, err := expressionNode.Operation.CandidateNode.Copy()
-		if err != nil {
-			return Context{}, err
-		}
+		clone := expressionNode.Operation.CandidateNode.Copy()
 		results.PushBack(clone)
 	}
 
